@@ -4,6 +4,7 @@
  * libmbim-glib -- GLib/GIO based library to control MBIM devices
  *
  * Copyright (C) 2013 - 2014 Aleksander Morgado <aleksander@aleksander.es>
+ * Copyright (C) 2022 Intel Corporation
  */
 
 #ifndef _LIBMBIM_GLIB_H_
@@ -25,6 +26,7 @@
 
 /* generated */
 #include "mbim-enum-types.h"
+#include "mbim-flag-types.h"
 #include "mbim-error-types.h"
 #include "mbim-basic-connect.h"
 #include "mbim-sms.h"
@@ -45,6 +47,11 @@
 #include "mbim-quectel.h"
 #include "mbim-intel-thermal-rf.h"
 #include "mbim-ms-voice-extensions.h"
+#include "mbim-intel-mutual-authentication.h"
+#include "mbim-intel-tools.h"
+#include "mbim-google.h"
+#include "mbim-fibocom.h"
+#include "mbim-compal.h"
 
 /* backwards compatibility */
 #include "mbim-compat.h"
