@@ -18,6 +18,44 @@
 #include "mbim-helpers.h"
 #include "mbim-error-types.h"
 
+/******************************************************************************/
+
+guint16
+mbim_helpers_read_unaligned_guint16 (const guint8 *buffer)
+{
+    guint16 tmp;
+
+    memcpy (&tmp, buffer, 2);
+    return GUINT16_FROM_LE (tmp);
+}
+
+guint32
+mbim_helpers_read_unaligned_guint32 (const guint8 *buffer)
+{
+    guint32 tmp;
+
+    memcpy (&tmp, buffer, 4);
+    return GUINT32_FROM_LE (tmp);
+}
+
+guint32
+mbim_helpers_read_unaligned_gint32 (const guint8 *buffer)
+{
+    gint32 tmp;
+
+    memcpy (&tmp, buffer, 4);
+    return GINT32_FROM_LE (tmp);
+}
+
+guint64
+mbim_helpers_read_unaligned_guint64 (const guint8 *buffer)
+{
+    guint64 tmp;
+
+    memcpy (&tmp, buffer, 8);
+    return GUINT64_FROM_LE (tmp);
+}
+
 /*****************************************************************************/
 
 gboolean
@@ -227,33 +265,3 @@ mbim_helpers_list_links_wwan (const gchar   *base_ifname,
     *out_links = g_steal_pointer (&links);
     return TRUE;
 }
-
-/*****************************************************************************/
-
-#if !GLIB_CHECK_VERSION(2,54,0)
-
-gboolean
-mbim_ptr_array_find_with_equal_func (GPtrArray     *haystack,
-                                     gconstpointer  needle,
-                                     GEqualFunc     equal_func,
-                                     guint         *index_)
-{
-  guint i;
-
-  g_return_val_if_fail (haystack != NULL, FALSE);
-
-  if (equal_func == NULL)
-      equal_func = g_direct_equal;
-
-  for (i = 0; i < haystack->len; i++) {
-      if (equal_func (g_ptr_array_index (haystack, i), needle)) {
-          if (index_ != NULL)
-              *index_ = i;
-          return TRUE;
-      }
-  }
-
-  return FALSE;
-}
-
-#endif
