@@ -1,23 +1,8 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 /*
  * libmbim-glib -- GLib/GIO based library to control MBIM devices
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA.
- *
  * Copyright (C) 2014 Aleksander Morgado <aleksander@lanedo.com>
  * Copyright (C) 2014 Smith Micro Software, Inc.
  */
@@ -42,6 +27,7 @@
 #include "mbim-enum-types.h"
 #include "mbim-error-types.h"
 #include "mbim-basic-connect.h"
+#include "mbim-ms-basic-connect-extensions.h"
 #include "mbim-proxy-helpers.h"
 
 /* The mbim-proxy may be used for bulk data transfer, such as modem
@@ -51,6 +37,10 @@
  */
 #define BUFFER_SIZE 4096
 
+/* The proxy control "Version" indication reporting the last agreed
+ * MBIMEx version, if any */
+#define MBIM_DEVICE_PROXY_CONTROL_VERSION "mbim-device-proxy-control-version"
+
 G_DEFINE_TYPE (MbimProxy, mbim_proxy, G_TYPE_OBJECT)
 
 enum {
@@ -186,6 +176,8 @@ client_unref (Client *client)
     }
 }
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (Client, client_unref)
+
 static Client *
 client_ref (Client *client)
 {
@@ -511,10 +503,6 @@ internal_open (GTask *task)
                       g_object_ref (self));
 }
 
-static void proxy_device_error_cb (MbimDevice *device,
-                                   GError     *error,
-                                   MbimProxy  *self);
-
 static void
 internal_device_open_caps_query_ready (MbimDevice   *device,
                                        GAsyncResult *res,
@@ -526,9 +514,6 @@ internal_device_open_caps_query_ready (MbimDevice   *device,
 
     self = g_task_get_source_object (task);
 
-    /* Always unblock error signals */
-    g_signal_handlers_unblock_by_func (device, proxy_device_error_cb, self);
-
     response = mbim_device_command_finish (device, res, &error);
     if (!response || !mbim_message_response_get_result (response, MBIM_MESSAGE_TYPE_COMMAND_DONE, &error)) {
         /* If we get a not-opened error, well, force closing right away and reopen */
@@ -575,11 +560,6 @@ internal_device_open (MbimProxy           *self,
     if (mbim_device_is_open (device)) {
         MbimMessage *message;
 
-        /* Avoid getting notified of errors in this internal check, as we're
-         * already going to check for the NotOpened error ourselves in the
-         * ready callback, and we'll reopen silently if we find this. */
-        g_signal_handlers_block_by_func (device, proxy_device_error_cb, self);
-
         g_debug ("[%s] checking device caps during client device open...",
                  mbim_device_get_path (device));
         message = mbim_message_device_caps_query_new (NULL);
@@ -604,8 +584,8 @@ process_internal_proxy_open (MbimProxy   *self,
                              Client      *client,
                              MbimMessage *message)
 {
-    Request *request;
-    MbimStatusError status = MBIM_STATUS_ERROR_FAILURE;
+    Request         *request;
+    MbimStatusError  status = MBIM_STATUS_ERROR_FAILURE;
 
     /* create request holder */
     request = request_new (self, client, message);
@@ -674,7 +654,8 @@ proxy_config_internal_device_open_ready (MbimProxy    *self,
                                          GAsyncResult *res,
                                          Request      *request)
 {
-    g_autoptr(GError) error = NULL;
+    g_autoptr(GError)  error = NULL;
+    MbimMessage       *indication;
 
     if (!internal_device_open_finish (self, res, &error)) {
         g_warning ("[client %lu,0x%08x] cannot configure proxy: couldn't open MBIM device: %s",
@@ -688,6 +669,15 @@ proxy_config_internal_device_open_ready (MbimProxy    *self,
     g_debug ("[client %lu,0x%08x] proxy configured",
              request->client->id, request->original_transaction_id);
 
+    /* notify the client about the MBIMEx version */
+    indication = (MbimMessage *) g_object_get_data (G_OBJECT (request->client->device), MBIM_DEVICE_PROXY_CONTROL_VERSION);
+    if (indication) {
+        if (!client_send_message (request->client, indication, &error))
+            g_warning ("[client %lu] couldn't report MBIMEx version update: %s", request->client->id, error->message);
+        else
+            g_debug ("[client %lu] reported MBIMEx version update", request->client->id);
+    }
+
     if (request->client->config_ongoing == TRUE)
         request->client->config_ongoing = FALSE;
     request->response = build_proxy_control_command_done (request->message, MBIM_STATUS_ERROR_NONE);
@@ -770,7 +760,7 @@ process_internal_proxy_config (MbimProxy   *self,
     }
 
     /* Retrieve path from request */
-    if (!_mbim_message_read_string (message, 0, 0, &incoming_path, &error)) {
+    if (!_mbim_message_read_string (message, 0, 0, MBIM_STRING_ENCODING_UTF16, &incoming_path, NULL, &error)) {
         g_warning ("[client %lu,0x%08x] cannot configure proxy: couldn't read device path from request: %s",
                    request->client->id, request->original_transaction_id, error->message);
         request->response = build_proxy_control_command_done (message, MBIM_STATUS_ERROR_INVALID_PARAMETERS);
@@ -973,6 +963,94 @@ process_device_service_subscribe_list (MbimProxy   *self,
     return TRUE;
 }
 
+/*****************************************************************************/
+/* MBIMEx version detection */
+
+static MbimMessage *
+build_proxy_control_version_notification (guint16 mbim_version,
+                                          guint16 ms_mbimex_version)
+{
+    MbimMessage                    *message;
+    struct indicate_status_message *indicate_status;
+    guint16                         tmp;
+    guint                           buffer_i = 0;
+    gsize                           buffer_length = 0;
+
+    buffer_length = sizeof (mbim_version) + sizeof (ms_mbimex_version);
+
+    message = (MbimMessage *) _mbim_message_allocate (MBIM_MESSAGE_TYPE_INDICATE_STATUS,
+                                                      0,
+                                                      sizeof (struct indicate_status_message) + buffer_length);
+    indicate_status = &(((struct full_message *)(message->data))->message.indicate_status);
+    indicate_status->fragment_header.total   = GUINT32_TO_LE (1);
+    indicate_status->fragment_header.current = 0;
+    memcpy (indicate_status->service_id, MBIM_UUID_PROXY_CONTROL, sizeof (MbimUuid));
+    indicate_status->command_id  = GUINT32_TO_LE (MBIM_CID_PROXY_CONTROL_VERSION);
+    indicate_status->buffer_length = GUINT32_TO_LE (buffer_length);
+
+    tmp = GUINT16_TO_LE (mbim_version);
+    memcpy (&indicate_status->buffer[buffer_i], &tmp, sizeof (tmp));
+    buffer_i += sizeof (tmp);
+    tmp = GUINT16_TO_LE (ms_mbimex_version);
+    memcpy (&indicate_status->buffer[buffer_i], &tmp, sizeof (tmp));
+    buffer_i += sizeof (tmp);
+    g_assert (buffer_i == buffer_length);
+
+    return message;
+}
+
+static void
+monitor_ms_basic_connect_extensions_version_response (MbimProxy   *self,
+                                                      MbimDevice  *device,
+                                                      MbimMessage *response)
+{
+    g_autoptr(MbimMessage)  indication = NULL;
+    guint16                 mbim_version;
+    guint16                 ms_mbimex_version;
+    guint8                  ms_mbimex_version_major;
+    guint8                  ms_mbimex_version_minor;
+    GList                  *l;
+
+    /* monitor the MBIMEx version agreed between the clients and the device */
+    if (!mbim_message_response_get_result (response, MBIM_MESSAGE_TYPE_COMMAND_DONE, NULL) ||
+        (mbim_message_command_done_get_service (response) != MBIM_SERVICE_MS_BASIC_CONNECT_EXTENSIONS) ||
+        (mbim_message_command_done_get_cid (response) != MBIM_CID_MS_BASIC_CONNECT_EXTENSIONS_VERSION) ||
+        !mbim_message_ms_basic_connect_extensions_v2_version_response_parse (response, &mbim_version, &ms_mbimex_version, NULL))
+        return;
+
+    ms_mbimex_version_major = ms_mbimex_version >> 8;
+    ms_mbimex_version_minor = ms_mbimex_version & 0xFF;
+
+    g_message ("Proxy monitoring detected MBIMEx version agreed with device: %x.%02x",
+               ms_mbimex_version_major, ms_mbimex_version_minor);
+    mbim_device_set_ms_mbimex_version (device, ms_mbimex_version_major, ms_mbimex_version_minor, NULL);
+
+    /* notify to all clients about the MBIMEx version update */
+    indication = build_proxy_control_version_notification (mbim_version, ms_mbimex_version);
+    for (l = self->priv->clients; l; l = g_list_next (l)) {
+        g_autoptr(GError)  error = NULL;
+        Client            *client;
+
+        client = l->data;
+        if (client->device != device)
+            continue;
+
+        if (!client_send_message (client, indication, &error))
+            g_warning ("[client %lu] couldn't report MBIMEx version update to %x.%02x: %s",
+                       client->id, ms_mbimex_version_major, ms_mbimex_version_minor, error->message);
+        else
+            g_debug ("[client %lu] reported MBIMEx version update to %x.%02x",
+                     client->id, ms_mbimex_version_major, ms_mbimex_version_minor);
+    }
+
+    /* the indication is stored as data associated to the device, so that it can
+     * be reused any time new clients attempt an open */
+    g_object_set_data_full (G_OBJECT (device),
+                            MBIM_DEVICE_PROXY_CONTROL_VERSION,
+                            mbim_message_ref (indication),
+                            (GDestroyNotify)mbim_message_unref);
+}
+
 /*****************************************************************************/
 /* Standard command */
 
@@ -1004,6 +1082,10 @@ device_command_ready (MbimDevice   *device,
     /* replace reponse transaction id with the requested transaction id */
     g_debug ("[client %lu,0x%08x] response from device received",
              request->client->id, request->original_transaction_id);
+
+    /* try to match the MBIMEx version exchange */
+    monitor_ms_basic_connect_extensions_version_response (request->self, device, request->response);
+
     mbim_message_set_transaction_id (request->response, request->original_transaction_id);
     request_complete_and_free (request);
 }
@@ -1098,37 +1180,39 @@ parse_request (MbimProxy *self,
 {
     do {
         g_autoptr(MbimMessage) message = NULL;
-        guint32                len = 0;
+        g_autoptr(GError)      error = NULL;
 
-        if (client->buffer->len >= sizeof (struct header) &&
-            (len = GUINT32_FROM_LE (((struct header *)client->buffer->data)->length)) > client->buffer->len) {
-            /* have not received complete message */
+        /* Invalid message? */
+        if (!_mbim_message_validate_internal ((const MbimMessage *)client->buffer, TRUE, &error)) {
+            /* No full message yet */
+            if (g_error_matches (error, MBIM_CORE_ERROR, MBIM_CORE_ERROR_INCOMPLETE_MESSAGE))
+                return;
+            /* Invalid message */
+            g_byte_array_remove_range (client->buffer, 0, client->buffer->len);
             return;
         }
 
-        if (!len)
-            return;
+        message = mbim_message_dup ((const MbimMessage *)client->buffer);
+        g_assert (message);
 
-        message = mbim_message_new (client->buffer->data, len);
-        if (!message)
-            return;
-
-        g_byte_array_remove_range (client->buffer, 0, len);
+        g_byte_array_remove_range (client->buffer, 0, mbim_message_get_message_length (message));
         process_message (self, client, message);
     } while (client->buffer->len > 0);
 }
 
 static gboolean
-connection_readable_cb (GSocket *socket,
-                        GIOCondition condition,
-                        Client *client)
+connection_readable_cb (GSocket      *socket,
+                        GIOCondition  condition,
+                        Client       *_client)
 {
+    g_autoptr(Client)  client = NULL;
     MbimProxy         *self;
     guint8             buffer[BUFFER_SIZE];
     g_autoptr(GError)  error = NULL;
     gssize             r;
 
     /* Recover proxy pointer soon */
+    client = client_ref (_client);
     self = client->self;
 
     if (condition & G_IO_HUP || condition & G_IO_ERR) {
@@ -1551,7 +1635,6 @@ dispose (GObject *object)
         if (g_socket_service_is_active (priv->socket_service))
             g_socket_service_stop (priv->socket_service);
         g_clear_object (&priv->socket_service);
-        g_unlink (MBIM_PROXY_SOCKET_PATH);
         g_debug ("UNIX socket service at '%s' stopped", MBIM_PROXY_SOCKET_PATH);
     }
 
