@@ -3,7 +3,7 @@
 /*
  * mbim-proxy -- A proxy to communicate with MBIM ports
  *
- * Copyright (C) 2014 Aleksander Morgado <aleksander@gnu.org>
+ * Copyright (C) 2014-2023 Aleksander Morgado <aleksander@aleksander.es>
  * Copyright (C) 2014 Smith Micro Software, Inc.
  */
 
@@ -34,6 +34,7 @@ static guint client_connected_once = FALSE;
 
 /* Main options */
 static gboolean verbose_flag;
+static gboolean verbose_full_flag;
 static gboolean version_flag;
 static gboolean no_exit_flag;
 static gint     empty_timeout = -1;
@@ -51,11 +52,15 @@ static GOptionEntry main_entries[] = {
       "Run action with verbose logs, including the debug ones",
       NULL
     },
+    { "verbose-full", 0, 0, G_OPTION_ARG_NONE, &verbose_full_flag,
+      "Run action with verbose logs, including the debug ones and personal info",
+      NULL
+    },
     { "version", 'V', 0, G_OPTION_ARG_NONE, &version_flag,
       "Print version",
       NULL
     },
-    { NULL }
+    { NULL, 0, 0, 0, NULL, NULL, NULL }
 };
 
 static gboolean
@@ -109,7 +114,7 @@ log_handler (const gchar    *log_domain,
         g_assert_not_reached ();
     }
 
-    if (!verbose_flag && !err)
+    if (!verbose_flag && !verbose_full_flag && !err)
         return;
 
     now = time ((time_t *) NULL);
@@ -129,7 +134,7 @@ print_version_and_exit (void)
 {
     g_print ("\n"
              PROGRAM_NAME " " PROGRAM_VERSION "\n"
-             "Copyright (C) 2013-2021 Aleksander Morgado\n"
+             "Copyright (C) 2013-2023 Aleksander Morgado\n"
              "Copyright (C) 2014-2018 Greg Suarez\n"
              "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl-2.0.html>\n"
              "This is free software: you are free to change and redistribute it.\n"
@@ -215,8 +220,16 @@ int main (int argc, char **argv)
 
     g_log_set_handler (NULL,  G_LOG_LEVEL_MASK, log_handler, NULL);
     g_log_set_handler ("Mbim", G_LOG_LEVEL_MASK, log_handler, NULL);
-    if (verbose_flag)
+    if (verbose_flag && verbose_full_flag) {
+        g_printerr ("error: cannot specify --verbose and --verbose-full at the same time\n");
+        exit (EXIT_FAILURE);
+    } else if (verbose_flag) {
         mbim_utils_set_traces_enabled (TRUE);
+        mbim_utils_set_show_personal_info (FALSE);
+    } else if (verbose_full_flag) {
+        mbim_utils_set_traces_enabled (TRUE);
+        mbim_utils_set_show_personal_info (TRUE);
+    }
 
     /* Setup signals */
     g_unix_signal_add (SIGINT,  quit_cb, NULL);
