@@ -4,6 +4,7 @@
  * mbimcli -- Command line interface to control MBIM devices
  *
  * Copyright (C) 2013 - 2014 Aleksander Morgado <aleksander@aleksander.es>
+ * Copyright (C) 2022 Intel Corporation
  */
 
 #include <glib.h>
@@ -12,6 +13,7 @@
 #define __MBIMCLI_H__
 
 #define VALIDATE_UNKNOWN(str) ((str) ? (str) : "unknown")
+#define VALIDATE_EMPTY(str) ((str) ? (str) : "")
 
 /* Common */
 void mbimcli_async_operation_done (gboolean operation_status);
@@ -30,6 +32,12 @@ GOptionGroup *mbimcli_quectel_get_option_group                     (void);
 GOptionGroup *mbimcli_intel_thermal_rf_get_option_group            (void);
 GOptionGroup *mbimcli_ms_voice_extensions_get_option_group         (void);
 GOptionGroup *mbimcli_ms_uicc_low_level_access_get_option_group    (void);
+GOptionGroup *mbimcli_intel_mutual_authentication_get_option_group (void);
+GOptionGroup *mbimcli_intel_tools_get_option_group                 (void);
+GOptionGroup *mbimcli_google_get_option_group                      (void);
+GOptionGroup *mbimcli_fibocom_get_option_group                     (void);
+GOptionGroup *mbimcli_sms_get_option_group                         (void);
+GOptionGroup *mbimcli_compal_get_option_group                      (void);
 
 gboolean      mbimcli_basic_connect_options_enabled               (void);
 gboolean      mbimcli_phonebook_options_enabled                   (void);
@@ -44,6 +52,12 @@ gboolean      mbimcli_quectel_options_enabled                     (void);
 gboolean      mbimcli_intel_thermal_rf_options_enabled            (void);
 gboolean      mbimcli_ms_voice_extensions_options_enabled         (void);
 gboolean      mbimcli_ms_uicc_low_level_access_options_enabled    (void);
+gboolean      mbimcli_intel_mutual_authentication_options_enabled (void);
+gboolean      mbimcli_intel_tools_options_enabled                 (void);
+gboolean      mbimcli_google_options_enabled                      (void);
+gboolean      mbimcli_fibocom_options_enabled                     (void);
+gboolean      mbimcli_sms_options_enabled                         (void);
+gboolean      mbimcli_compal_options_enabled                      (void);
 
 void          mbimcli_basic_connect_run                 (MbimDevice   *device,
                                                          GCancellable *cancellable);
@@ -71,7 +85,18 @@ void          mbimcli_ms_voice_extensions_run           (MbimDevice   *device,
                                                          GCancellable *cancellable);
 void          mbimcli_ms_uicc_low_level_access_run      (MbimDevice   *device,
                                                          GCancellable *cancellable);
-
+void          mbimcli_intel_mutual_authentication_run   (MbimDevice   *device,
+                                                         GCancellable *cancellable);
+void          mbimcli_intel_tools_run                   (MbimDevice   *device,
+                                                         GCancellable *cancellable);
+void          mbimcli_google_run                        (MbimDevice   *device,
+                                                         GCancellable *cancellable);
+void          mbimcli_fibocom_run                       (MbimDevice   *device,
+                                                         GCancellable *cancellable);
+void          mbimcli_sms_run                           (MbimDevice *device,
+                                                         GCancellable *cancellable);
+void          mbimcli_compal_run                        (MbimDevice *device,
+                                                         GCancellable *cancellable);
 
 /* link management */
 GOptionGroup *mbimcli_link_management_get_option_group (void);
